window.Gleam.initSearch([{"doc":"formal","title":"formal","content":"# formal\n\nType safe HTML form decoding and validation!\n\n[![Package Version](https://img.shields.io/hexpm/v/formal)](https://hex.pm/packages/formal)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/formal/)\n\n```sh\ngleam add formal\n```\n```gleam\nimport formal/form\n\n// Define a type that is to be decoded from the form data\npub type SignUp {\n  SignUp(email: String, password: String)\n}\n\n// This function takes the list of key-value string pairs that a HTML form\n// produces. It then decodes the form data into a SignUp value, ensuring that\n// all the fields are present and valid.\n//\npub fn handle_form_submission(values: List(#(String, String))) {\n  let result = \n    form.decoding({\n      use email &lt;- form.parameter\n      use password &lt;- form.parameter\n      SignUp(email: email, password: password)\n    })\n    |&gt; form.with_values(values)\n    |&gt; form.field(\n      &quot;email&quot;,\n      form.string\n        |&gt; form.and(form.must_not_be_empty)\n        |&gt; form.and(form.must_be_an_email),\n    )\n    |&gt; form.field(\n      &quot;password&quot;,\n      form.string\n        |&gt; form.and(form.must_not_be_empty)\n        |&gt; form.and(form.must_be_string_longer_than(7))\n    )\n    |&gt; form.finish\n\n  case result {\n    Ok(data) -&gt; {\n      // Do something with the SignUp value here\n    }\n    Error(form_state) -&gt; {\n      // Re-render the form with the error messages\n    }\n  }\n}\n```\n\nAn example of using this library with Wisp and Lustre can be found in\n[./example/](https://github.com/lpil/formal/tree/main/example).\n\nFurther documentation can be found at &lt;https://hexdocs.pm/formal&gt;.\n","url":"index.html"},{"doc":"formal/form","title":"Form","content":"pub type Form {\n  Form(\n    values: Dict(String, List(String)),\n    errors: Dict(String, String),\n  )\n}\n An invalid or unfinished form. This is either created by the `new`\n function, which creates a new empty form, or by the `finish` function when\n the validation failed, returning the invalid form.\n\nForm(\n  values: Dict(String, List(String)),\n  errors: Dict(String, String),\n)\n\n\nSynonyms:\nformal/form.Form\nformal/form Form","url":"formal/form.html#Form"},{"doc":"formal/form","title":"FormValidator","content":"pub opaque type FormValidator(output)\n A collection of validations that decode data from a form into a typed value.\n\n See the module documentation for an overview of how to use this type to\n validate a form.\n\n\nSynonyms:\nformal/form.FormValidator\nformal/form FormValidator","url":"formal/form.html#FormValidator"},{"doc":"formal/form","title":"and","content":"pub fn and(\n  previous: fn(a) -&gt; Result(b, String),\n  next: fn(b) -&gt; Result(c, String),\n) -&gt; fn(a) -&gt; Result(c, String)\n Add an additional validation step to the field decoder function.\n\n This function behaves similar to the `try` function in the standard library\n `gleam/result` module.\n\nSynonyms:\nformal/form.and\nformal/form and","url":"formal/form.html#and"},{"doc":"formal/form","title":"bool","content":"pub fn bool(input: String) -&gt; Result(Bool, String)\n Decode the field value as a bool.\n\n The input value must be &quot;on&quot; to be considered true as this is the value\n that HTML checkboxes use when checked.\n\n # Examples\n\n \n bool(&quot;on&quot;)\n # -&gt; Ok(True)\n \n\n \n bool(&quot;true&quot;)\n # -&gt; Ok(False)\n \n\n \n bool(&quot;&quot;)\n # -&gt; Ok(False)\n \n\nSynonyms:\nformal/form.bool\nformal/form bool","url":"formal/form.html#bool"},{"doc":"formal/form","title":"decoding","content":"pub fn decoding(\n  into constructor: fn(a) -&gt; b,\n) -&gt; FormValidator(fn(a) -&gt; b)\n Set the constructor that is used to create the success value if the form\n decodes and validates successfully.\n\n You will want to use a curried constructor function here. The `curry*`\n functions in the `gleam/function` standard library module can be used to\n help with this.\n\nSynonyms:\nformal/form.decoding\nformal/form decoding","url":"formal/form.html#decoding"},{"doc":"formal/form","title":"field","content":"pub fn field(\n  form: FormValidator(fn(a) -&gt; b),\n  name: String,\n  decoder: fn(String) -&gt; Result(a, String),\n) -&gt; FormValidator(b)\n Add the next field to be decoded and validated from the form, corresponding\n to the next argument to the constructor.\n\nSynonyms:\nformal/form.field\nformal/form field","url":"formal/form.html#field"},{"doc":"formal/form","title":"field_state","content":"pub fn field_state(\n  form: Form,\n  name: String,\n) -&gt; Result(Nil, String)\n Check the field in a `FieldState` for an error, returning it as the `Error`\n variant of a `Result` if it exists.\n\n This function may be helpful when rendering a HTML form.\n\nSynonyms:\nformal/form.field_state\nformal/form field_state","url":"formal/form.html#field_state"},{"doc":"formal/form","title":"finish","content":"pub fn finish(form: FormValidator(a)) -&gt; Result(a, Form)\n Finish the form validation, returning the success value built using the\n constructor, or the invalid form containing the values and errors, which\n can be used to render the form again to the user.\n\nSynonyms:\nformal/form.finish\nformal/form finish","url":"formal/form.html#finish"},{"doc":"formal/form","title":"float","content":"pub fn float(input: String) -&gt; Result(Float, String)\n Decode the field value as a float.\n\n The input value must have a decimal point.\n\n # Examples\n\n \n float(&quot;12.34&quot;)\n # -&gt; Ok(123)\n \n\n \n float(&quot;1&quot;)\n # -&gt; Error(&quot;Must be a number with a decimal point&quot;)\n \n\n \n float(&quot;ok&quot;)\n # -&gt; Error(&quot;Must be a number with a decimal point&quot;)\n \n\nSynonyms:\nformal/form.float\nformal/form float","url":"formal/form.html#float"},{"doc":"formal/form","title":"initial_values","content":"pub fn initial_values(values: List(#(String, String))) -&gt; Form\n Create a new form with some initial values.\n\n You likely want to use this or `new` when rendering a page\n containing a new form.\n\nSynonyms:\nformal/form.initial_values\nformal/form initial_values","url":"formal/form.html#initial_values"},{"doc":"formal/form","title":"int","content":"pub fn int(input: String) -&gt; Result(Int, String)\n Decode the field value as an int.\n\n # Examples\n\n \n int(&quot;123&quot;)\n # -&gt; Ok(123)\n \n\n \n int(&quot;ok&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \n\nSynonyms:\nformal/form.int\nformal/form int","url":"formal/form.html#int"},{"doc":"formal/form","title":"list","content":"pub fn list(\n  of decoder: fn(String) -&gt; Result(a, String),\n) -&gt; fn(List(String)) -&gt; Result(List(a), String)\n Decode all the values for a field as a given type. This is useful with the\n `multifield` function when there are multiple inputs with the same name in\n the form.\n\n # Examples\n\n \n int(&quot;123&quot;)\n # -&gt; Ok(123)\n \n\n \n int(&quot;ok&quot;)\n # -&gt; Error(&quot;Must be a whole number&quot;)\n \n\nSynonyms:\nformal/form.list\nformal/form list","url":"formal/form.html#list"},{"doc":"formal/form","title":"message","content":"pub fn message(\n  result: fn(a) -&gt; Result(b, String),\n  message: String,\n) -&gt; fn(a) -&gt; Result(b, String)\n Set a custom error message for a field decoder, overwriting the previous\n error message if there is one at this point in the decoder, doing nothing\n if the decoder is successful.\n\nSynonyms:\nformal/form.message\nformal/form message","url":"formal/form.html#message"},{"doc":"formal/form","title":"multifield","content":"pub fn multifield(\n  form: FormValidator(fn(a) -&gt; b),\n  name: String,\n  decoder: fn(List(String)) -&gt; Result(a, String),\n) -&gt; FormValidator(b)\n Add the next field to be decoded and validated from the form, corresponding\n to the next argument to the constructor.\n\n This function is useful when you have multiple inputs with the same name in\n the form, and you most likely want to use it with the `list` decoder\n function. When there is only a single input with the given name in the form\n then the `field` function is more appropriate.\n\nSynonyms:\nformal/form.multifield\nformal/form multifield","url":"formal/form.html#multifield"},{"doc":"formal/form","title":"must_be_accepted","content":"pub fn must_be_accepted(input: Bool) -&gt; Result(Bool, String)\n Assert that the bool input is true. This is expected to be used with\n checkboxes and has an error message that reflects that.\n\n # Examples\n\n \n must_be_accepted(True)\n # -&gt; Ok(True)\n \n\n \n must_be_accepted(False)\n # -&gt; Error(&quot;Must be accepted&quot;)\n \n\nSynonyms:\nformal/form.must_be_accepted\nformal/form must_be_accepted","url":"formal/form.html#must_be_accepted"},{"doc":"formal/form","title":"must_be_an_email","content":"pub fn must_be_an_email(input: String) -&gt; Result(String, String)\n Assert that the string input looks like an email address.\n\n It could still be an invalid email address even if it looks like one. To\n validate an email address is valid you will need to send an email to it and\n ensure your user receives it.\n\n # Examples\n\n \n must_be_an_email(&quot;hello@example.com&quot;)\n # -&gt; Ok(&quot;hello@example.com&quot;)\n \n\n \n must_be_an_email(&quot;Something&quot;)\n # -&gt; Error(&quot;Must be an email&quot;)\n \n\nSynonyms:\nformal/form.must_be_an_email\nformal/form must_be_an_email","url":"formal/form.html#must_be_an_email"},{"doc":"formal/form","title":"must_be_greater_float_than","content":"pub fn must_be_greater_float_than(\n  minimum: Float,\n) -&gt; fn(Float) -&gt; Result(Float, String)\n Assert that the float input is greater than the given minimum.\n\n It could still be an invalid email address even if it looks like one. To\n validate an email address is valid you will need to send an email to it and\n ensure your user receives it.\n\n # Examples\n\n \n let check = must_be_greater_float_than(3.3)\n check(4.1)\n # -&gt; Ok(3.3)\n \n\n \n let check = must_be_greater_float_than(3.3)\n check(2.0)\n # -&gt; Error(&quot;Must be greater than 3.3&quot;)\n \n\nSynonyms:\nformal/form.must_be_greater_float_than\nformal/form must_be_greater_float_than","url":"formal/form.html#must_be_greater_float_than"},{"doc":"formal/form","title":"must_be_greater_int_than","content":"pub fn must_be_greater_int_than(\n  minimum: Int,\n) -&gt; fn(Int) -&gt; Result(Int, String)\n Assert that the int input is greater than the given minimum.\n\n It could still be an invalid email address even if it looks like one. To\n validate an email address is valid you will need to send an email to it and\n ensure your user receives it.\n\n # Examples\n\n \n let check = must_be_greater_int_than(10)\n check(12)\n # -&gt; Ok(12)\n \n\n \n let check = must_be_greater_int_than(10)\n check(2)\n # -&gt; Error(&quot;Must be greater than 10&quot;)\n \n\nSynonyms:\nformal/form.must_be_greater_int_than\nformal/form must_be_greater_int_than","url":"formal/form.html#must_be_greater_int_than"},{"doc":"formal/form","title":"must_be_lesser_float_than","content":"pub fn must_be_lesser_float_than(\n  maximum: Float,\n) -&gt; fn(Float) -&gt; Result(Float, String)\n Assert that the float input is greater than the given minimum.\n\n It could still be an invalid email address even if it looks like one. To\n validate an email address is valid you will need to send an email to it and\n ensure your user receives it.\n\n # Examples\n\n \n let check = must_be_lesser_float_than(10)\n check(12)\n # -&gt; Ok(12)\n \n\n \n let check = must_be_lesser_float_than(10)\n check(2)\n # -&gt; Error(&quot;Must be less than 10&quot;)\n \n\nSynonyms:\nformal/form.must_be_lesser_float_than\nformal/form must_be_lesser_float_than","url":"formal/form.html#must_be_lesser_float_than"},{"doc":"formal/form","title":"must_be_lesser_int_than","content":"pub fn must_be_lesser_int_than(\n  maximum: Int,\n) -&gt; fn(Int) -&gt; Result(Int, String)\n Assert that the int input is greater than the given minimum.\n\n It could still be an invalid email address even if it looks like one. To\n validate an email address is valid you will need to send an email to it and\n ensure your user receives it.\n\n # Examples\n\n \n let check = must_be_lesser_int_than(10)\n check(12)\n # -&gt; Ok(12)\n \n\n \n let check = must_be_lesser_int_than(10)\n check(2)\n # -&gt; Error(&quot;Must be less than 10&quot;)\n \n\nSynonyms:\nformal/form.must_be_lesser_int_than\nformal/form must_be_lesser_int_than","url":"formal/form.html#must_be_lesser_int_than"},{"doc":"formal/form","title":"must_be_string_longer_than","content":"pub fn must_be_string_longer_than(\n  length: Int,\n) -&gt; fn(String) -&gt; Result(String, String)\n Assert that the string is longer than the given length.\n\n # Examples\n\n \n let check = must_be_string_longer_than(4)\n check(&quot;hello&quot;)\n // -&gt; Ok(&quot;hello&quot;)\n \n\n \n let check = must_be_string_longer_than(2)\n check(&quot;hi&quot;)\n // -&gt; Error(&quot;Must be longer than 2 characters&quot;)\n \n\nSynonyms:\nformal/form.must_be_string_longer_than\nformal/form must_be_string_longer_than","url":"formal/form.html#must_be_string_longer_than"},{"doc":"formal/form","title":"must_be_string_shorter_than","content":"pub fn must_be_string_shorter_than(\n  length: Int,\n) -&gt; fn(String) -&gt; Result(String, String)\n Assert that the string is shorter than the given length.\n\n # Examples\n\n \n let check = must_be_string_shorter_than(3)\n check(&quot;hi&quot;)\n // -&gt; Ok(&quot;hi&quot;)\n \n\n \n let check = must_be_string_shorter_than(5)\n check(&quot;hello&quot;)\n // -&gt; Error(&quot;Must be shorter than 5 characters&quot;)\n \n\nSynonyms:\nformal/form.must_be_string_shorter_than\nformal/form must_be_string_shorter_than","url":"formal/form.html#must_be_string_shorter_than"},{"doc":"formal/form","title":"must_equal","content":"pub fn must_equal(\n  expected: a,\n  because error_message: String,\n) -&gt; fn(a) -&gt; Result(a, String)\n Assert that the bool input is true. This is expected to be used with\n checkboxes and has an error message that reflects that.\n\n # Examples\n\n \n let check = must_equal(42, &quot;Must be the answer to everything&quot;)\n check(42)\n # -&gt; Ok(42)\n \n\n \n let check = must_equal(42, &quot;Must be the answer to everything&quot;)\n check(2)\n # -&gt; Error(&quot;Must be the answer to everything&quot;)\n \n\nSynonyms:\nformal/form.must_equal\nformal/form must_equal","url":"formal/form.html#must_equal"},{"doc":"formal/form","title":"must_not_be_empty","content":"pub fn must_not_be_empty(input: String) -&gt; Result(String, String)\n Assert that the string input must not be empty, returning an error if it\n is.\n\n # Examples\n\n \n must_not_be_empty(&quot;Hello&quot;)\n # -&gt; Ok(&quot;Hello&quot;)\n \n\n \n must_not_be_empty(&quot;&quot;)\n # -&gt; Error(&quot;Must not be blank&quot;)\n \n\nSynonyms:\nformal/form.must_not_be_empty\nformal/form must_not_be_empty","url":"formal/form.html#must_not_be_empty"},{"doc":"formal/form","title":"new","content":"pub fn new() -&gt; Form\n Create a new empty form.\n\n You likely want to use this or `initial_values` when rendering a page\n containing a new form.\n\nSynonyms:\nformal/form.new\nformal/form new","url":"formal/form.html#new"},{"doc":"formal/form","title":"number","content":"pub fn number(input: String) -&gt; Result(Float, String)\n Decode the field value as a float.\n\n The decimal point is optional.\n\n # Examples\n\n \n number(&quot;12.34&quot;)\n # -&gt; Ok(123)\n \n\n \n number(&quot;1&quot;)\n # -&gt; Ok(1.0)\n \n\n \n number(&quot;ok&quot;)\n # -&gt; Error(&quot;Must be a number&quot;)\n \n\nSynonyms:\nformal/form.number\nformal/form number","url":"formal/form.html#number"},{"doc":"formal/form","title":"parameter","content":"pub fn parameter(f: fn(a) -&gt; b) -&gt; fn(a) -&gt; b\n This function is used to create constructor functions that take arguments\n one at a time, making them suitable for passing to the `decode` function.\n\n # Examples\n\n \n form.decoding({\n   use name &lt;- parameter\n   use email &lt;- parameter\n   SignUp(name: name, email: email)\n })\n |&gt; form.with_values(values)\n |&gt; form.field(&quot;email&quot;, string)\n |&gt; form.field(&quot;password&quot;, string)\n |&gt; form.finish\n \n\nSynonyms:\nformal/form.parameter\nformal/form parameter","url":"formal/form.html#parameter"},{"doc":"formal/form","title":"string","content":"pub fn string(input: String) -&gt; Result(String, String)\n Decode the field value as a string.\n\n # Examples\n\n \n string(&quot;hello&quot;)\n # -&gt; Ok(&quot;hello&quot;)\n \n\nSynonyms:\nformal/form.string\nformal/form string","url":"formal/form.html#string"},{"doc":"formal/form","title":"value","content":"pub fn value(form: Form, name: String) -&gt; String\n Get a single value from a `Form`, returning an empty string if there\n was no value. If there was multiple values for the field name then the\n first is returned.\n\n This function may be helpful for getting values for inputs when rendering a\n HTML form.\n\n If you want a `Result` back instead use the `dict.get` function with\n `form.values`.\n\nSynonyms:\nformal/form.value\nformal/form value","url":"formal/form.html#value"},{"doc":"formal/form","title":"with_values","content":"pub fn with_values(\n  form: FormValidator(a),\n  values: List(#(String, String)),\n) -&gt; FormValidator(a)\n Set the values from the form submission to be validated.\n\n HTML forms can have multiple fields with the same name. This function will\n use the final value with each name, so if you wish to use a different value\n consider removing the duplicates or using `with_values_dict`.\n\nSynonyms:\nformal/form.with_values\nformal/form with_values","url":"formal/form.html#with_values"},{"doc":"formal/form","title":"with_values_dict","content":"pub fn with_values_dict(\n  form: FormValidator(a),\n  values: Dict(String, List(String)),\n) -&gt; FormValidator(a)\n Set the values from the form submission to be validated.\n\nSynonyms:\nformal/form.with_values_dict\nformal/form with_values_dict","url":"formal/form.html#with_values_dict"},{"doc":"formal/form","title":"formal/form","content":"","url":"formal/form.html"}]);